/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void SurfaceWaveOp::initialize(void)
{
	catalog<String>("Attribute")="P";

	catalog<Real>("Scalar")=0.9;
	catalog<Real>("Scalar","low")=0.1;
	catalog<Real>("Scalar","high")=10.0;
	catalog<Real>("Scalar","min")=0.01;
	catalog<Real>("Scalar","max")=100.0;

	catalog<SpatialVector>("Offset")=SpatialVector(1.2, 3.4, 5.6);

	catalog<bool>("Test Toggle")=true;

	catalog<I32>("Test Int")=7;
	catalog<String>("Test Int","IO")="output";

//	catalog<Vector4>("Test Vector4")=Vector4(0.1, 0.2, 0.3, 0.7);

	//* The default 'implementation' is an appropriate SurfaceAccessibleI.
	catalog< sp<Component> >("Input Surface");

	//* This "Output" entry is essentially redundant, as given, since an
	//* auto-copy output will be made by default if no output is explicitly
	//* specified, but the explicit definition does add a presence to the
	//* Catalog.  Otherwise, it would only appear in the signal.
	catalog< sp<Component> >("Output Surface");
	catalog<String>("Output Surface","IO")="output";
	catalog<String>("Output Surface","copy")="Input Surface";
}

void SurfaceWaveOp::handle(Record& a_rSignal)
{
//	feLog("SurfaceWaveOp::handle\n");

	const String& attribute=catalog<String>("Attribute");
	const Real scalar=catalog<Real>("Scalar");
	const SpatialVector& offset=catalog<SpatialVector>("Offset");
	I32& rTestInt=catalog<I32>("Test Int");
	const bool testToggle=catalog<bool>("Test Toggle");

	if(testToggle)
	{
		if(++rTestInt>7)
		{
			rTestInt=0;
		}
	}

	feLog("  Attribute: %s\n",attribute.c_str());
	feLog("  Scalar: %.6G\n",scalar);
	feLog("  Offset: %s\n",c_print(offset));
	feLog("  Test Toggle: %d\n",testToggle);
	feLog("  Test Int: %d\n",rTestInt);
//	feLog("  Test Vector4: %s\n",c_print(catalog<Vector4>("Test Vector4")));

	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	feLog("SurfaceWaveOp::handle spSurfaceAccessibleI valid=%d\n",
			spSurfaceAccessibleI.isValid());
	FEASSERT(spSurfaceAccessibleI.isValid());

	I32 frame=0;			// TODO
	Real time=0.03*frame;

	sp<SurfaceAccessorI> spAccessor=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,attribute);
	FEASSERT(spAccessor.isValid());

	U32 count=spAccessor->count();
	feLog("SurfaceWaveOp::handle count=%d\n",count);

	for(U32 index=0;index<count;index++)
	{
		SpatialVector point=spAccessor->spatialVector(index);

		point[1]+=scalar*sin(point[0]*0.2 + point[2]*0.3 + time);
		point+=offset;

		spAccessor->set(index,point);
	}

//	feLog("SurfaceWaveOp::handle done\n");
}

} // namespace
