/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __thread_Gang_h__
#define __thread_Gang_h__

namespace fe
{

/**************************************************************************//**
	@brief Group of Worker threads

	@ingroup thread
*//***************************************************************************/
template<typename WORKER,typename T>
class Gang: public JobQueue<T>
{
	public:
				Gang(void)
				{	Counted::setName("Gang"); }
virtual			~Gang(void)
				{
					if(JobQueue<T>::m_workers)
					{
						finish();
					}
				}

		void	start(U32 threads,sp<Counted> spObject=sp<Counted>(NULL))
				{
					FEASSERT(!JobQueue<T>::m_workers);
					FEASSERT(threads>0);

					JobQueue<T>::m_workers=threads;
					sp< JobQueue<T> > spJobQueue(this);

					m_worker=new WORKER*[threads];

					for(U32 m=0;m<threads;m++)
					{
						m_worker[m]=new WORKER(m,spJobQueue);
						m_worker[m]->setObject(spObject);
					}

					for(U32 m=0;m<threads;m++)
					{
//						boost::thread* pThread=
								m_group.create_thread(*m_worker[m]);
					}
				}

		void	finish(void)
				{
					FEASSERT(JobQueue<T>::m_workers);

					m_group.join_all();

					for(U32 m=0;m<JobQueue<T>::m_workers;m++)
					{
						delete m_worker[m];
					}

					delete[] m_worker;
					JobQueue<T>::m_workers=0;

					//* empty out queue
					T job;
					while(take(job)){}
				}

	private:
		boost::thread_group	m_group;
		WORKER**			m_worker;
};

} // namespace

#endif // __thread_Gang_h__
